#!/usr/bin/env python

# Reads the entire database and produces a summary of students and their
# progress, graphically.
##############################################################################

import cgitb
cgitb.enable()
from tasks import read_all_tasks
from global_vars import *
import datetime
from get_form_info import get_form
import sys
import os
import re

all_tasks = dict((t.id, t) for t in read_all_tasks())
for task in all_tasks.itervalues():
    task.set_owner()
    task.set_acquire_time() 

def nice_time(minutes):
    return "%d:%02d" % (minutes / 60, minutes % 60)
   
def get_report_start_time():
    form = get_form()
    result = []
    for v, d in [('start_year', 2009),
                 ('start_month', 10),
                 ('start_day', 2),
                 ('start_hour', 11),
                 ('start_minute', 00)]:
        try:
            result.append(int(form[v].value))
        except:
            result.append(d)
    return datetime.datetime(*result)

def get_report_interval():
    form = get_form()
    try:
        v = int(form['interval'].value)
    except:
        v = 5
    return datetime.timedelta(minutes=v)

def get_tick_count():
    form = get_form()
    try:
        return int(form['tick_count'].value)
    except:
        return 60
    return datetime.timedelta(minutes=v)

start_time = get_report_start_time()
report_interval = get_report_interval()
tick_count = get_tick_count()

# collect all logs:
log_entries = []
regexp = re.compile(r'(..)/(..)/(....) (..):(..):(..)')

class LogEntry(object):
    def __init__(self, line):
        line = line.strip().split()
        if line[1] == '+':
            self.sign = 1
        else:
            self.sign = -1
        self.id = line[2]
        (mo, d, y, h, m, s) = (int(i) for i in
                               regexp.match(' '.join(line[3:5])).groups())
        self.acquire_time = datetime.datetime(y,mo,d,h,m,s)
        self.user = line[0]

for (d, sd, files) in os.walk('task_data/users'):
    for n in files:
        if not n.endswith('_log.txt'):
            continue
        f = file(os.path.join(d, n))
        log_entries.extend(LogEntry(l) for l in f)
log_entries.sort(key=lambda key:key.acquire_time)

if len(log_entries) == 0:
    print "Content-type: text/plain\n"
    print "No one has done anything yet!"
    sys.exit(0)

log_epoch = start_time
while log_entries[0].acquire_time <= log_epoch:
    log_epoch -= report_interval
current_time = log_epoch

total_times = {}
taken_times = {}
lines = {}

for i in xrange(1,9):
    total_times['cat_%d' % i] = sum(t.duration_in_minutes
                                for t in all_tasks.itervalues()
                                if int(t.category) == i)
    taken_times['cat_%d' % i] = 0
for f in file('task_data/usersfile.txt'):
    total_times[f.split()[0]] = 22 * 60
    taken_times[f.split()[0]] = 0

img = list("<img src='/imgs/%d.png'>" % i for i in xrange(11))
log_index = 0
time_tick = 0

while log_index < len(log_entries) and time_tick < tick_count:
    c = log_entries[log_index]
    if current_time < c.acquire_time:
        # next task on log is ahead of current time
        # move current time forward, 
        if start_time < current_time:
            # but only writing tasks to lines
            # if we're inside the drawing interval
            for (tt_id, tt) in taken_times.iteritems():
                v = lines.get(tt_id, "") 
                fraction = int(tt / (total_times[tt_id] / 10.0))
                if 10 < fraction: fraction = 10
                v = v + img[fraction]
                lines[tt_id] = v
            time_tick += 1
        current_time += report_interval
    else:
        # update current allocations for the current log
        d = c.sign * all_tasks[c.id].duration_in_minutes
        if c.user in taken_times:
            taken_times[c.user] += d
            taken_times['cat_'+c.id[0]] += d
        log_index += 1

while time_tick < tick_count:
    for (tt_id, tt) in taken_times.iteritems():
        v = lines.get(tt_id, "") 
        fraction = int(tt / (total_times[tt_id] / 10.0))
        if 10 < fraction: fraction = 10
        v = v + img[fraction]
        lines[tt_id] = v
    time_tick += 1
        
print 'Content-type: text/html\n'
print '<html>'
print '<head><link rel="stylesheet" href="/styles/pas_style.css"></head><body>'

print '<h1>Categories</h1><table>'
for c in range(1,9):
    print '<tr><td width=150> Category',c,'</td><td>'
    ix = 'cat_%d' % c
    print lines[ix]
    print '</td><td>%s/%s' % (nice_time(taken_times[ix]),
                              nice_time(total_times[ix]))
    print '</td></tr>'
print '</table>'

print '<h1>Student Volunteers</h1><table>'
for userline in file('task_data/usersfile.txt'):
    ix = userline.split()[0]
    print '<tr><td width=150>',ix,'</td><td>'
    print lines[ix]
    print '</td><td>%s/%s' % (nice_time(taken_times[ix]),
                              nice_time(total_times[ix]))
    print '</td></tr>'
print '</table>'

print '</body></html>'
