#!/usr/bin/env python

# Reads the entire database and produces a summary of students and their
# tasks
##############################################################################

import cgitb
cgitb.enable()
from tasks import read_all_tasks
from global_vars import *
import display_progress

tasks = dict((t.id, t) for t in read_all_tasks())

for task in tasks.itervalues():
    task.set_owner()
    
task_count = len(tasks)
taken_tasks = sum(1 for t in tasks.itervalues() if not t.available)
total_time = sum(t.duration_in_minutes for t in tasks.itervalues())
taken_time = sum(t.duration_in_minutes for t in tasks.itervalues()
                 if not t.available)

def nice_time(minutes):
    return "%d:%02d" % (minutes / 60, minutes % 60)

def ascii_time_to_int(t):
    (h, m) = list(int(i) for i in t.split(':'))
    return h * 60 + m

print "Content-type: text/html\n"
print '<html><head><title>Task summary</title><link rel="stylesheet" href="/styles/pas_style.css"></head><body>'
print "<h1>Overview</h1>"
print "%s/%s tasks assigned" % (taken_tasks, task_count)
print "<br> %s / %s time assigned" % (nice_time(taken_time),
                                      nice_time(total_time))
print "<h2>SV summary</h2>"

svs = {}

for task in tasks.itervalues():
    if task.owner:
        if task.owner not in svs:
            svs[task.owner] = []
        svs[task.owner].append(task)

sv_list = sorted(svs.items(),
                 key=lambda ((owner, tasks)): sum(task.duration_in_minutes
                                                  for task in tasks))
users = {}
def print_sv_summary():
    for f in file('task_data/usersfile.txt'):
        username = f.split()[0]
        users[f.split()[0]] = (f.split()[2] + ' ' + f.split()[3])
        if username not in svs:
            print ("0:00 / 22:00 <a href='#%s'>%s</a> (<a href='activity_summary.py?name=%s'>log</a>, <a href='user_task.py?name=%s'>individual task list</a>)" %
                   (username,username,username,username))
            print " (%s)" % users[f.split()[0]]
            print '<br>'
    for (owner, tasks) in sv_list:
        print ("%s / 22:00 <a href='#%s'>%s</a> <a href='activity_summary.py?name=%s'>log</a> <a href='user_task.py?name=%s'>individual task list</a>)" %
               (nice_time(sum(task.duration_in_minutes
                              for task in tasks)), owner, owner, owner, owner))
        print " (%s)" % users[owner]
        print '<br>'
print_sv_summary()

print "<h2>Open tasks</h2>"
print "<table>"
print "<tr><th>id</th><th>date</th><th>start</th><th>end</th><th>name</th></tr>"
for task in sorted(tasks.values(), key=lambda t: (t.date, t.start, t.id)):
    if task.available:
        print task.dump_tr(['id', 'date', 'start', 'end', 'name'])
print "</table>"

print "<h2>Detailed SV schedule</h2>"
def print_detailed_schedule():
    for f in file('task_data/usersfile.txt'):
        user, last = f.split()[:2]
        print "<div class='Heading'><a name='%s'><b>%s</b></a></div>" % (user, user)
        dp = display_progress.display_progress(user, last)
        print "<table><tr><td valign=top>"
        dp.print_tasks(False, sort_key=lambda k: (k[2], ascii_time_to_int(k[4]))) # 2 is date, 4 is start time
        print "</td><td valign=top>"
        dp.print_table()
        print "</td></tr></table>"
        if user not in svs:
            print "<center>"
            for i in xrange(1,9):
                print "Cat %d: <b>0:00</b>" % i
                if i < 8: print " - "
            print "</center>"
            continue
        per_cat = [0,0,0,0,0,0,0,0,0,0]
        for task in svs[user]:
            per_cat[int(task.category)] += task.duration_in_minutes
        print "<center>"
        for cat in xrange(1,9):
            print "Cat. %d: <b>%s</b>" % (cat, nice_time(per_cat[cat]))
            if cat < 8: print " - "
        print "</center>"
        print "<hr>"
print_detailed_schedule()

print "</body></html>"
