from global_vars import *
from utils import print_error
import os.path
import re
import datetime

def py_to_js_bool(v):
    if v:
        return 'true'
    else:
        return 'false'

class Task(object):
    def __init__(self, line):
        (self.id,
         self.category,
         self.date,
         self.dayofweek,
         self.start,
         self.end,
         self.duration,
         self.name,
         self.location) = line.strip().split('\t')
        task_filename = (task_checked_folder +
                         self.category +
                         '/%s.txt' % self.id)
        self.available = not os.path.isfile(task_filename)
        hours, minutes = self.duration.split(':')
        self.duration_in_minutes = int(hours) * 60 + int(minutes)
    def dump_js(self):
        return ('new task("%s", "%s", "%s", "%s", "%s", "%s", "%s", "%s", "%s", %s)' %
                (self.id, self.category, self.date, self.dayofweek,
                 self.start, self.end, self.duration, self.name,
                 self.location,
                 py_to_js_bool(self.available)))
    def dump_tr(self, items, html_attrs={}):
        def get_html_attrs(t):
            try:
                return html_attrs[t]
            except KeyError:
                return ""
        def get_text(t):
            try:
                return getattr(self, t)
            except AttributeError:
                return " "
        return '<tr>' + ''.join('<td %s>%s</td>' % (get_html_attrs(t), get_text(t))
                                for t in items) + '</tr>'
    def set_owner(self):
        # we don't set owner by default to not hit the filesystem as often.
        if self.available:
            self.owner = None
        else:
            f = file(task_checked_folder +
                     self.category +
                     '/%s.txt' % self.id)
            self.owner = f.readline().split()[0]

    def set_acquire_time(self):
        if self.available:
            self.acquire_time = None
        else:
            f = file(task_checked_folder +
                     self.category +
                     '/%s.txt' % self.id)
            regexp = re.compile(r'.+ (..)/(..)/(....) (..):(..):(..)')
            (mo, d, y, h, m, s) = (int(i) for i in
                                   regexp.match(f.readline().strip()).groups())
            self.acquire_time = datetime.datetime(y,mo,d,h,m,s)

def read_all_tasks():
    try:
        f = file(all_tasks_filename)
    except:
        report_error('could not open tasks\' file', 1)
        return []
    result = []
    for line in f:
        try:
            line[0]
        except:
            continue
        if line[0] == '#':
            continue
        result.append(Task(line[:-1]))
    f.close()
    return result
