#!/usr/bin/env python

##############################################################################
# cleanup_files.py
# 
# Cleans up the database, and creates a zipfile of the previous state.
#
# Warning: you have to run this as root, so BE EXTRA CAREFUL.
#
# Needs to run as root because the cgi-bin files are being written by
# some other user (www or www-data). We could do this better, but won't.
#

import os
import os.path
import sys
import time
from pwd import getpwnam

file_owner = 'www-data'
owner_uid = getpwnam(file_owner).pw_uid
owner_gid = getpwnam(file_owner).pw_gid

if os.getuid() != 0:
    print "This will only work as root"
    sys.exit(1)

# First, we check if we are in the directory we should be.
if not os.path.realpath(os.path.curdir).endswith('task_data'):
    print "Please run this from the task_data directory"
    sys.exit(1)
if not os.path.isdir('tasks_checked'):
    print "ERROR:"
    print "Could not find tasks_checked directory - file structure corrupt?"
    sys.exit(1)
if not os.path.isdir('users'):
    print "ERROR:"
    print "Could not find users directory - file structure corrupt?"
    sys.exit(1)

files_to_erase = []

# collect all checked-out task logs
for (dirname, subdirs, file_names) in os.walk('tasks_checked'):
    for f in file_names:
        files_to_erase.append(os.path.join(dirname, f))

# collect all user task lists and logs
for (dirname, subdirs, file_names) in os.walk('users'):
    for f in file_names:
        files_to_erase.append(os.path.join(dirname, f))

if len(files_to_erase) == 0:
    print "Database empty - nothing to do!"
    sys.exit(0)
    
outname = time.strftime("backup_%Y%m%d_%H%M%S.zip")
    

# save all files to zip
for n in files_to_erase:
    if os.system('zip %s %s' % (outname, n)) != 0:
        print "ERROR creating backup."
        sys.exit(1)

if os.system('unzip -t %s' % outname) != 0:
    print "ERROR: backup is corrupted"
    sys.exit(1)

os.chown(outname, owner_uid, owner_gid)


if len(sys.argv) > 1 and sys.argv[1] == '-n':
    print "Backup OK! Will exit without deleting."
    sys.exit(0)

print "Backup OK! Will delete now."

for n in files_to_erase:
    os.unlink(n)

print "Done: backup file is %s" % outname
