#!/usr/bin/env python

import sys, string
import display_tasks
import portalocker
import cgitb; cgitb.enable()
import get_form_info
import nice_display

from time import time, strftime, localtime

from global_vars import *

class validate_taskID:
  def __init__ (self):
    # get info from for (task ID, user, user's last name)
    self.get_info_from_form()
    # using task ID get category, date, time (start, finish, duration), and description
    self.get_task_info()
    # get all the tasks already in from the user (create a file if no tasks has been entered)
    # also, checks the new task is not in user's list already
    self.get_user_info()
    # check for scheduling conflicts with tasks already in the user's list
    self.check_schedule()
    # finally, check that the task is available (i.e. no other user has check it out)
    self.check_task_availability()
    # if all is fine, create file for task (that's the way to check if task's already taken)
    # and append the new task to user's list
    self.write_task()
    # redraw everything
    self.end()

  def file_exist (self, filename):
    # Jsut to check is the file exist. A (module) os could have been used, but this
    # might be faster
    try:
      input = open (filename, 'r')
    except:
      return 0  # does not exist
    input.close()
    return 1    # it exists

  def write_task (self):
    # define paths and names for the task (such as 120.txt, 338.txt) 
    # where 120 and 338 are task numbers
    path_category = 'task_data/tasks_checked/category_'+ self.task_info[1] + '/'
    task_log_fn = path_category + 'log.txt'
    task_ID_fn  = path_category + self.task_info[0] + '.txt'
    # The easiest check: if file exist, then the task is taken
    if self.file_exist (task_ID_fn):
      self.error ('(116) task %s has been checked out.' % self.task_info[0], 2)
    # define paths and filenames for users (such as Alejandro_tl.txt, Robert_tl.txt)
    # where Alejandro and Robert are users and the file contains all the tasks assigned to 
    # that person.
    user_log_fn = users_path + self.user + '_log.txt'
    user_task_list_fn = users_path + self.user + '_tl.txt'
    # Since the task does not seem taken, first check the lock out for tasks
    try: # check if lock if off (i.e. not taken)
      task_log = open(task_log_fn, "a+")
    except: # lock on, go back and let user try again
      self.error ("(102) Could not create lock, system might be busy,\\n wait a few seconds and try again.",2)
    # lock the file to assure Mutual exclusion (mutex)
#    try:
#      portalocker.lock(task_log, portalocker.LOCK_EX)
#    except: # lock on, go back and let user try again
#      self.error ("(103) System is busy, wait a few seconds and try again.", 2)

    # second, look for user's lock. Again, this would ensure mutex
    try:
      user_log = open(user_log_fn, "a+")
    except: # lock on, go back and let user try again
      task_log.close() # also RELEASE the task lock (very important)
      self.error ("(104) User's file is busy (probably used in another window). \\nIf you think this is a mistake", 1)
#    try: # user's file is open. Now needs to be locked
#      portalocker.lock(user_log, portalocker.LOCK_EX)
#    except:
#      task_log.close() # also RELEASE the task lock (very important)
#      self.error ("(105) User's file is busy (probably used in another window). \\nIf you think this is a mistake", 1)
      
    # if here, then both log files are opened and locked
    # open user's config file (e.g. Alejandro_tl.txt)
    try:
      user_task_list = open (user_task_list_fn, 'a+')
    except:
      user_log.close()
      task_log.close()
      self.error ('(106) There was a problem with user\'s task file', 1)
    # then create task file (e.g. 120.txt)
    try:
      task_ID = open (task_ID_fn, 'w')
    except:
      task_log.close()
      user_log.close()
      self.error ('(107) There is a problem with the lock (could not create task ID)', 1)
    # 4 files are opened. First, write the new task in the user's task list file
    for i in range (len(self.task_info)-1):
      user_task_list.write('%s\t' % self.task_info[i])
    user_task_list.write ('%s\n' % self.task_info[i+1][:-1])
    user_task_list.close () # not needed anymore. 3 more opened.
    # get local time
    timestamp = strftime(" %m/%d/%Y %H:%M:%S\n", localtime(time()))
    # write user and time in the task ID file (e.g 120.txt)
    task_ID.write(self.user + timestamp)
    task_ID.close() # it's not needed anymore. 2 more opened.
    # write the timestamp in the log (just for control)
    user_log.write (self.user + ' + ' + self.task_info[0] + timestamp)
    user_log.close() # 1 more opened
    # write who and when the task file was acccessed (or tried to be)
    task_log.write (self.user + ' + ' + self.task_info[0] + timestamp)
    task_log.close() # everything is closed


  def check_task_availability (self):
    filename = 'tasks_checked/category_'+ self.task_info[1] + '/' + self.task_info[0] + '.txt'
    try:
      input = open (filename, 'r')
    except:
      return
    self.error ('(108) task %s has been checked out.' % self.task_info[0], 2)

    
  def get_user_info (self):
    filename = users_path + self.user + '_tl.txt'
    try:
      input = open (filename, 'r')
    except:
      try:
        input = open (filename,'w')
      except:
        self.error ('(109) Could not open user file', 1)
      self.user_list = []
      return
    self.user_list = []
    for line in input.readlines():
      strtok = line.split('\t')
      try: 
        strtok[0]
      except:
        continue
      if strtok[0][0] == '#':
        continue
      self.user_list.append (strtok)
    input.close()
    
      
  def error (self, text, type):
    # send it to the front page to display it as js alert
    nice_display.display_it_all(type, text)
    # quit this module

  def get_info_from_form(self):
    try:
      self.form=get_form_info.get_form()
    except: # It should happen, but just in case
      self.error ('(110) Error in getting info from form', 1)
      return
    # get all the important values from user's form
    self.user = self.form['user'].value
    self.last_name = self.form['last_name'].value
    self.taskID = string.strip (self.form['taskID'].value)
    self.ensure_user_exists()

  def ensure_user_exists(self):
    for l in file('task_data/usersfile.txt'):
      f = l.split()[0]
      if f == self.user:
        return
    self.error('(666) Incorrect username or password.', 2)
    
  def check_schedule (self):
    # rename the task info (just to reduce typing)
    ti = self.task_info
    same_day = []  # init all tasks that are in the same day
    # Check if task is already in user's list
    for ul in self.user_list:
      if  ti[0] == ul [0]: # Check that task is not in user's list. Actually, this is not
                           # necessary because it should've been flagged as task checked before
        self.error ('(111) Task %s is already on your list.' % ul[0], 2)
      # Create list of all the tasks for the same day
      if ti[2] == ul[2]: # tasks happens in the same day than the new one
        same_day.append (ul)
    # init total hours with the duration time from the new task
    if ti[1] == '5':
      total_dur = self.convert2num (ti[6])
      for ul in self.user_list: # go through the whole list
        if ul[1] == '5': # Check if it is in category 5
          dur = self.convert2num (ul[6])
          total_dur = total_dur + dur
#      if total_dur > MAX_NUM_CAT_5:
#        self.error ('Task %s cannot be added because\\nsurpasses the time limits for Category 5.' % ti[1], 2)
      # Check if there is no scheduling conflict with tasks already selected
    if not len(same_day) :
      # no task the same day, then return. Everything is dandy
      return
    # If here, more than one task in the same day.
    # Get new task (as string) and convert it (to numbers)
    tok = ti[4].split (':')
    tok2 = ti[5].split(':')
    temp = (string.atof(tok[0]), string.atof(tok[1]),\
           string.atof(tok2[0]), string.atof(tok2[1]))
    # nt contains the begin and end hour 
    # (e.g. (12.25, 13.25), which means starts at 12:15 pm and ends 1:15 pm)
    nt = (temp[0] + temp[1]/60.0, temp[2] + temp[3]/60.0)
    for sd in same_day:
      # Get new task (as string) and convert it (to numbers)
      tok = sd[4].split (':')
      tok2 = sd[5].split(':')
      temp = (string.atoi(tok[0]), string.atoi(tok[1]),\
             string.atoi(tok2[0]), string.atoi(tok2[1]))
      # ot contains the begin and end hour 
      # (e.g. (12.75, 15.25), which means starts at 12:45 pm and ends 3:15 pm)
      ot = (temp[0] + temp[1]/60.0, temp[2] + temp[3]/60.0)
      # Check if starting time is in conflict
      # first check if start or end time is in between
      # (e.g old task is 12:15 - 13:35, and new task is 13:30 - 14:30)
      if (ot[0] < nt[0] < ot[1]) or (ot[0] < nt[1] < ot[1]):
        self.error ('(112) Task %s has a scheduling conflict with task %s.' % (ti[0],sd[0]), 2)
      # then check if already in task is contained in new task
      # (e.g old task is 12:15 - 15:45, and new task is 13:00 - 14:30)
      if (nt[0] <= ot[0]) and (nt[1] >= ot[1]):
        self.error ('(113) Task %s has a scheduling conflict with task %s.' % (ti[0],sd[0]), 2)
    # if it get to this point -> no conflicts
    # just checking if category 5 is not surpassed (5 because is the one most interesting to work in)
#    return # not anymore. There are plenty of Cat 5 tasks
    #if ti[1] <> '5':
    #  return # not in Category 5
#    else:
#      print 'total_dur ', total_dur
#      print 'MAX ', MAX_NUM_CAT_5

  def convert2num (self, hour_str):
    tok = hour_str.split (':')
    temp = (string.atof(tok[0]), string.atof(tok[1]))
    ctn = temp[0] + temp[1]/60.0
    return ctn

  def get_task_info (self):
    try:
      input = open (all_tasks_filename, 'r')
    except:
      self.error ('(114) Could not open all tasks file', 1)
    self.task_info = []
    for line in input.readlines ():
      strtok = line.split('\t')
      try: 
        strtok[0]
      except:
        continue
      if strtok[0][0] == '#':
        continue # coment, skip it
      if strtok[0] == self.taskID:
        for s in strtok:
          self.task_info.append (s)
        return
    self.error ('(115) Could not find task %s.' % self.taskID, 2)
      
  def end (self, msg_num = 0, msg_str=""):
    nice_display.display_it_all(msg_num, msg_str)
    
# main
validate_taskID()
